'''
__author__="P.Korniliev"
__date__ ="$Feb 10, 2015 5:10:13 PM$"
COMMON LIBRARY for running GWAS associations. Cornell 2015.
FUNCTIONS:
init() 
    loads all parameters from appconfig.ini file 
check() 
    shows major settings loaded from config file

prep_files():
  Prepares files for GWAS:
     1. Makes pheno files for each sub-population specified in config file matched to geno files
     2. Creates and fiters genome files for each sub-population specified in config file
     3. Makes kinship matrix file and covariate file if requested
runGWA():      
     1. Checks if all needed files exist
     2. Runs GWA using EMMAX mixed model
     2. Makes QQ and MP for each trait
     3. Makes summary table for the set of most significant SNPs

gChrom(snp) 
    given SNP name returns a chromosome name
gPos(snp)
    given SNP name returns a bp position
getEdgesOrdered: 
    creates temp files for MPs
makeMPfinal:
    makes temp files for building MP
getGenes:
    given a gene data file name return a list of genes
makeMPfinalColored():
    makes temp files for building MP
makeSinglePheno():
    given a phenotype index creates a temp single phenotype file
makeSinglePhenoFile(fName, index, tName):
    given a phenotype index creates a temp single phenotype file in the project directory using the 
makeSinglePhenoFileList(fName, index, tName):
    given a phenotype index creates a temp single phenotype file in the project directory using the 
makeSinglePhenoFileList_ExH(fName, index, tName):
    given a phenotype index creates a temp single phenotype file in the project directory using the 
    trait name (tName) in file name, drops the header
run_EMMAX():
    run emmax for one phenotype using covariate file that could be ommited
getFiles(strLoc,strExt):
    given a location and extension of the file returns a list of files
getSuffixDict(fName):
    gets a list of headers from the features file
runCommand(command)
    runs system command
getPositions(fName):
    given a PLINK map file, returns a dictionary of SNPs [chromosome, bp position]
getHeaders(fName):
    given a phenotype file name returns a list of phenotype headers
createGeneSNPIndxFile(posFile,geneFile,foutName):
    given a SNPs positions file (preliminary for MP built with real positions before adjustments)
filterVar(fName):
    given *.tped file name filters out non-variant SNPs and leaves only those that passed MAC filter
gSNP_Allele_info():
    using SNPs informationfile creates a dictionary of SNPs where the key is SNP name, first element - chromosome, second - position
    A - allele, B - allele
checkPemmaxLimit(fName, pLimit):
    given a result file and pValue limit, returns a list of sorted by pValue SNPs with a pValue elements that satisfy limit condition
extractPhenoFreqDta(lsSNPs, lTraitData, dSNP, GFile, MFile):
    #gets Allele frequencies and mean values for each of the SNP in the list
'''
CONFIG_FILE_NAME = 'appconfig.ini'
GENO_LOC=""   #Plink files location
PHENO_LOC="" #Phenotype file location
R_OUT_LOC =""  #Results file location
RES_SUM_FILE=""
EMM_LOC=""
PROJ=""
GFILE=""
POPFILE=""
POPS=["ALL"]
GROUPS=["RDP12"]
MAC_FILTER=0
MAF_FILTER=""
GENO_FILTER=""
PLINK=""
RScript4PC=""
SNPsDATA=""
OVERWRITE=True
FDR_LIMIT=0.1
BEST_SNP_LIMIT=0.1
QQplotLambda=1
QQplotStdError=1
P_FILTER=1

import time, gzip, sys, os
import numpy as np

def init():
    '''
    loads all parameters from appconfig.ini file
    '''
    global GENO_LOC   #Plink files location
    global PHENO_LOC #Phenotype file location
    global R_OUT_LOC  #Results file location
    global PLOT_LOC #Plots location
    global RES_SUM_FILE
    global EMM_LOC
    global PROJ
    global POPS
    global GROUPS
    global GFILE
    global POPFILE
    global PHENO_FILE
    global GT_VER
    global MAC_FILTER
    global PLINK
    global MAF_FILTER
    global GENO_FILTER
    global MATRIX #type of kinship matrix - IBS or BN
    global PC  #number of principal components as covariates
    global RScript4PC # R script to build PC co-variate files and build plots
    global SNPsDATA
    global OVERWRITE
    global FDR_LIMIT
    global BEST_SNP_LIMIT
    global mkQQplot
    global mkMPlot
    global QQplotLambda
    global QQplotStdError
    
    if os.path.exists(CONFIG_FILE_NAME):
        import ConfigParser
        Config = ConfigParser.ConfigParser()
        Config.read(CONFIG_FILE_NAME)

        GENO_LOC=Config.get('GenomeData','GLOC')
        GFILE=Config.get('GenomeData','GFILE')
        GT_VER=Config.get('GenomeData','GT_VER')
        POPFILE=Config.get('GenomeData','POPFILE')
        SNPsDATA=Config.get('GenomeData','SNPsDATA')

        PROJ=Config.get('ProjectLocation','PROJ')
        R_OUT_LOC="%sresults/"%PROJ
        PLOT_LOC="%splots/"%PROJ

        PHENO_LOC=Config.get('PhenoData','PLOC')
        PHENO_FILE=Config.get('PhenoData','PHENO_FILE')

        EMM_LOC=Config.get('Software','EMM_LOC')
        PLINK=Config.get('Software','PLINK')
        RScript4PC=Config.get('Software','RScript4PC')

        POPS=Config.get('RunParameters','POPS').split(" ")
        GROUPS=Config.get('RunParameters','GROUPS').split(" ")
        MAC_FILTER=int(Config.get('RunParameters','MAC_FILTER'))
        MAF_FILTER=Config.get('RunParameters','MAF_FILTER')
        GENO_FILTER=Config.get('RunParameters','GENO_FILTER')
        MATRIX=Config.get('RunParameters','MATRIX')
        PC=Config.get('RunParameters','PC')
        OVERWRITE=Config.get('RunParameters','OVERWRITE')
        FDR_LIMIT=Config.get('RunParameters','FDR_LIMIT')
        BEST_SNP_LIMIT=Config.get('RunParameters','BEST_SNP_LIMIT')
        mkQQplot=Config.get('RunParameters','mkQQplot')
        mkMPlot=Config.get('RunParameters','mkMPlot')
        QQplotLambda=Config.get('RunParameters','QQplotLambda')
        QQplotStdError=Config.get('RunParameters','QQplotLambda')
    else:
        print "%s configuration file was not found"%CONFIG_FILE_NAME

def check():
    '''
    shows major settings loaded from config file
    '''
    print "GENO_LOC:%s\nPHENO_LOC:%s\nR_OUT_LOC:%s\nRES_SUM_FILE:%s\nEMM_LOC:%s\nPROJ:%s\n"%(GENO_LOC,PHENO_LOC,R_OUT_LOC,RES_SUM_FILE,EMM_LOC,PROJ)
    print POPS
def gChrom(snp):
    '''
    given SNP name returns a chromosome name
    '''
    #return snp.split('c')[1].split('p')[0]
    return snp.split('.')[0]

def gPos(snp):
    '''
    given SNP name returns a bp position
    '''
    #return int(snp.split('p')[-1])
    return int(snp.split('.')[-1])

def _getEdgesOrdered(dataFile,base,res_loc,p_filter,dPos,header=False,pIndx=2,sInd=0):
    '''
    creates temp files for MPs
    params:
    name of result file
    name base for temp result files
    '''
    #print dataFile
    dOut=dict()
    df = open(dataFile,'r')
    pf = open('%s%s_prelim'%(res_loc,base),'w')
    ef = open('%s%s_edges'%(res_loc,base),'w')
    fAxOut = '%saxes_%s'%(res_loc,base)
    fAx=open(fAxOut,'w')
    chrm = ''
    iCount=0
    begin = True
    indx = pIndx
    indSNP = sInd
    iLastLoc = 0
    head = header
    lstChroms = list()
    for i in df: 
        l = i.split() 
        if head:
            head=False
            indx = l.index('P')
            indSNP = l.index('SNP')
            #print "indx:%i  indSNP:%i"%(indx,indSNP)
            #print i
        else:
         #print l
         if l[indx] != 'NA':
            pVal=float(l[indx])
         else:
            pVal=1
    
         if pVal<=p_filter and pVal>0:
            #cr=gChrom(l[indSNP])
            #pos=gPos(l[indSNP])
            cr=dPos[l[indSNP]][0]
            pos=dPos[l[indSNP]][1]
            pf.write('%s\t%i\t%e\n'%( cr, pos, pVal))
            if cr <> chrm and not begin:
                if not chrm in dOut:
                    dOut[chrm]=['0']*2
                    iCount+=1
                dOut[chrm][1] = iLastLoc
                chrm = cr
                if not chrm in dOut:
                    dOut[chrm]=['0']*2
                dOut[chrm][0]=pos
                iLastLoc = pos 
                print chrm
                lstChroms.append(chrm)
            elif not begin:
                iLastLoc = pos
            else:
                chrm = cr
                if not chrm in dOut:
                    dOut[chrm]=['0']*2
                dOut[chrm][0]=pos
                begin = False
                print chrm
                lstChroms.append(chrm)
    dOut[chrm][1] = iLastLoc 
    df.close()
    pf.close()
    #print dOut
    slid=0
    print 'getEdgesOrdered: iCount=%i'%len(dOut)
    for j in range(len(dOut)):
        if lstChroms[j] in dOut:
            print '%s\t%i\t%i\n'%(lstChroms[j],dOut[lstChroms[j]][0],dOut[lstChroms[j]][1])
            ef.write('%s\t%i\t%i\n'%(lstChroms[j],dOut[lstChroms[j]][0],dOut[lstChroms[j]][1]))
            chrome=lstChroms[j]
            fAx.write('%s\t%i\n'%(chrome,slid))
            slid=slid+dOut[lstChroms[j]][1]-dOut[lstChroms[j]][0]+1
    fAx.write('end\t%i\n'%(slid))        
    ef.close()
    fAx.close()
    return dOut,len(dOut),fAxOut

def makeMPfinal(base,dCorr,res_loc):
    '''
    makes temp files for building MP
    '''
    iCount=0
    fOutName = '%sMP_%s'%(res_loc,base)
    #fOutAxes = '%saxes_%s'%(res_loc,base)
    fI=open('%s%s_prelim'%(res_loc,base),'r')
    fO=open(fOutName,'w')
    corrFact=0
    run=0 #last position for previous chromosome
    chrome=""
    for i in fI:
        l=i.split()
        if chrome!=str(l[0]):
            corrFact = run - dCorr[l[0]][0] +1
            chrome=str(l[0])
        run=int(l[1]) + corrFact
        fO.write('%s\t%i\t%s\n'%(chrome,run,l[2]))
        iCount+=1
    fI.close()
    fO.close()
    return fOutName,iCount

def getEdgesOrdered(dataFile,base,res_loc,p_filter,dSNP): 
  #creates temp files for MPs
  #params:
  #   name of result file
  #   name base for temp result files
  
  print dataFile
  dOut=dict()
  df = open(dataFile,'r')
  pf = open('%s%s_prelim'%(res_loc,base),'w')
  ef = open('%s%s_edges'%(res_loc,base),'w')
  fAxOut = '%saxes_%s'%(res_loc,base)
  fAx=open(fAxOut,'w')
  chr = ''
  iCount=0
  begin = True
  iSum = 0 
  iLen = 0
  indx = 2 
  for i in df: 
    l = i.split()  
    if l[indx] != 'NA' and l[indx]>0:
        pVal=float(l[indx])
    else:
        pVal=1
    
    if pVal<=p_filter and pVal>0:
        #cr=gChrom(l[0])
        #pos=gPos(l[0])
        cr=dSNP[l[0]]['chrom']
        pos=dSNP[l[0]]['pos']
        pf.write('%s\t%i\t%e\n'%( cr, pos, pVal))
        if cr <> chr and not begin:
            if not chr in dOut:
                dOut[chr]=['0']*2
                iCount+=1
            dOut[chr][1] = iLastLoc
            chr = cr
            if not chr in dOut:
                dOut[chr]=['0']*2
            dOut[chr][0]=pos
            iLastLoc = pos 
            print chr
        elif not begin:
            iLastLoc = pos
        else:
            chr = cr
            if not chr in dOut:
                dOut[chr]=['0']*2
            dOut[chr][0]=pos
            begin = False
            print chr
  dOut[chr][1] = iLastLoc 
  df.close()
  pf.close()
  print dOut
  slid=0
  print 'getEdgesOrdered: iCount=%i'%len(dOut)
  for j in range(1,len(dOut)+1):
      if str(j) in dOut:
          print '%s\t%i\t%i\n'%(str(j),dOut[str(j)][0],dOut[str(j)][1])
          ef.write('%s\t%i\t%i\n'%(str(j),dOut[str(j)][0],dOut[str(j)][1]))
          chrome=str(j)
          fAx.write('%s\t%i\n'%(chrome,slid))
          slid=slid+dOut[str(j)][1]-dOut[str(j)][0]+1
  fAx.write('end\t%i\n'%(slid))        
  ef.close()
  fAx.close()
  return dOut,len(dOut),fAxOut

def getGenes(fName):
    ''' 
    given a gene data file name return a list of genes
    '''
    lGenes=list()
    if fName=="NA":
        return lGenes
    else:
       fIn=open(fName,'r')
       header=True
       for i in fIn:
           if header:
               header =False
               continue
           l=i.split()
           chrom = l[1]
           start = int(l[2])
           finish = int(l[3])
           lGenes.append([chrom,start,finish])
    fIn.close()
    return lGenes

def makeMPfinalColored(base,dCorr,res_loc,CFName,P_Limit):
    '''
    makes temp files for building MP
    CFName - candidate genes file name // could be "NA" if no data available
    P_Limit - the p-value limit after which the color of the SNP will be changed
    in the last column - the color code added:
    1 - standard SNP
    2 - SNP with better than P_Limit value
    3 - SNP that satisfy #2 + lay in limits of the genes provided in Candidate Genes file
    '''
    #fist get a genes list of interest
    lGenes=getGenes(CFName)
    iCount=0
    fOutName = '%sMP_%s'%(res_loc,base)
    #fOutAxes = '%saxes_%s'%(res_loc,base)
    fI=open('%s%s_prelim'%(res_loc,base),'r')
    fO=open(fOutName,'w')
    corrFact=0
    run=0 #last position for previous chromosome
    chrome=""
    for i in fI:
        l=i.split()
        colCode = "1" #default color code for standard SNP
        if chrome!=str(l[0]):
            corrFact = run - dCorr[l[0]][0] +1
            chrome=str(l[0])
        run=int(l[1]) + corrFact
        if float(l[2])<P_Limit:
            colCode = "2"
            #check genes
            for g in lGenes:
                if g[0] == chrome:
                    if int(l[1])>= g[1] and  int(l[1])<=g[2]:
                        colCode = "3"
        fO.write('%s\t%i\t%s\t%s\n'%(chrome,run,l[2],colCode))
        iCount+=1
    fI.close()
    fO.close()
    return fOutName,iCount

def makeSinglePheno(fName, index):
    '''
    given a phenotype index creates a temp single phenotype file
    pheno_temp.txt
    '''
    fTOut = "pheno_temp.txt"
    fIn=open(fName,'r')
    fOut = open(fTOut,'w')
    for i in fIn:
        l=i.split()
        fOut.write("%s\t%s\t%s\n"%(l[0],l[1],l[index+2]))
    fOut.close()
    fIn.close()
    return fTOut

def makeSinglePhenoFile(fName, index, tName):
    '''
    given a phenotype index creates a temp single phenotype file in the project directory using the 
    trait name (tName) in file name
    pheno_temp.txt
    '''
    fTOut = "%spheno_%s.txt"%(PLOC,tName)
    fIn=open(fName,'r')
    fOut = open(fTOut,'w')
    for i in fIn:
        l=i.split()
        fOut.write("%s\t%s\t%s\n"%(l[0],l[1],l[index+2]))
    fOut.close()
    fIn.close()
    return fTOut

def makeSinglePhenoFileList(fName, index, tName):
    '''
    given a phenotype index creates a temp single phenotype file in the project directory using the 
    trait name (tName) in file name
    pheno_temp.txt
    '''
    lOut=list()
    fTOut = "%spheno_%s.txt"%(PLOC,tName)
    fIn=open(fName,'r')
    fOut = open(fTOut,'w')
    for i in fIn:
        l=i.split()
        fOut.write("%s\t%s\t%s\n"%(l[0],l[1],l[index+2]))
        lOut.append(l[index+2])
    fOut.close()
    fIn.close()
    return fTOut,lOut

def makeSinglePhenoFileList_ExH(fName, index, tName):
    '''
    given a phenotype index creates a temp single phenotype file in the project directory using the 
    trait name (tName) in file name, drops the header
    '''
    lOut=list()
    fTOut = "%spheno_%s.txt"%(PROJ,tName)
    fIn=open(fName,'r')
    fOut = open(fTOut,'w')
    header = True
    for i in fIn:
        if header:
            header=False
        else:
            l=i.split()
            fOut.write("%s\t%s\t%s\n"%(l[0],l[1],l[index+2]))
            lOut.append(l[index+2])
    fOut.close()
    fIn.close()
    return fTOut,lOut

def run_EMMAX(geno_file,pheno_file, ibd_file, base_out, cov_file="NA"):
    '''
    run emmax for one phenotype using covariate file that could be ommited
    returns a name of results file
    '''
    if cov_file=="NA":
        cov_param=""
    else:
        cov_param="-c %s"%cov_file 
    runCommand("%semmax -v -d 10 -t %s -p %s %s -k %s -o %s"%(EMM_LOC,geno_file,pheno_file, cov_param, ibd_file,base_out))

def getFiles(strLoc,strExt):
    '''
    given a location and extension of the file returns a list of files
    '''
    import filecmp as fc
    d = fc.dircmp(strLoc,strLoc)
    lOut=list()
    total_list=d.left_list
    for i in total_list:
        if i.endswith(strExt):
            lOut.append(i)
    return lOut    

def getSuffixDict(fName):
    '''
    gets a list of headers from the features file
    '''
    dOut=dict()
    lOut=list()
    fIn=open(fName,'r')
    l=fIn.readline().strip().split()
    for j,k in enumerate(l):
        if j>1:
            dOut[k]='P%i'%(j-1)
            lOut.append(k)
    return dOut,lOut

def runCommand(command):
    sys.stdout.write('\t\t%s\n%s\n%s\n' %(time.asctime(), command, '-'*len(command)))
    sys.stdout.flush()
    if os.system(command):
        raise OSError
    
def getPositions(fName):
    '''
    given a PLINK map file, returns a dictionary of SNPs [chromosome, bp position]
    '''
    dOut=dict()
    for i in open('%s%s.map'%(GENO_LOC,fName),'r'):
        l=i.split()
        dOut[l[1]]=[l[0],l[3]]
    return dOut 
  
def getHeaders(fName):
    '''
    given a phenotype file name returns a list of phenotype headers
    '''
    f=open('%s'%fName,'r')
    l=f.readline()
    ls=l.split()
    assert (ls[0]=='FID' and ls[1]=='IID'), 'WRONG phenotype file structure!!!'
    lOut=list()
    for j,k in enumerate(ls):
        if j>1:
            lOut.append(k.strip())
    f.close()
    return lOut

def createGeneSNPIndxFile(posFile,geneFile,foutName):
    '''
    given a SNPs positions file (preliminary for MP built with real positions before adjustments),
    candidate genes file name outPut file name 
    creates file of the gene names and the index of the closest SNP found in the list
    it will be either the first SNP in gene location or the closest one
    '''

    fIn=open(geneFile,'r')
    fBase=open(posFile,'r')
    fOut = open(foutName,'w')
    header=True
    indx=0

    snpLinePrev=""
    snpLine=""
    for i in fIn:
        if header:
            header=False
            continue
        else:
            l=i.split()
            gene = l[4].strip()[1:-1]
            chrom = l[1]
            bpS = int(l[2])
            bpF = int(l[3])
            snpLinePrev=snpLine
            snpLine=fBase.readline()
            s=snpLine.split()
            runChr=s[0]
            runPos=int(s[1])
            indx+=1
            
            #first roll to the correct chromosome 
            if  int(runChr)<int(chrom):
                while int(runChr)<int(chrom):
                    snpLinePrev=snpLine
                    snpLine=fBase.readline()
                    s=snpLine.split()
                    runChr=s[0]
                    runPos=int(s[1])
                    indx+=1
            #find the first SNP on or beyond the gene start location
            while runChr==chrom and bpS>runPos:
                snpLinePrev=snpLine
                snpLine=fBase.readline()
                s=snpLine.split()
                runChr=s[0]
                runPos=int(s[1])
                indx+=1
                
            #current SNP is in the next chromosome already - use the previous one
            if runChr!=chrom and int(runChr)<int(chrom):
                fOut.write("%s %i\n"%(gene,indx-1))
            #current SNP is inside the gene location 
            elif bpS<=runPos and bpF>runPos:
                fOut.write("%s %i\n"%(gene,indx))
            elif bpF<runPos:
                preRunPos=int(snpLinePrev.split()[1])
                if (runPos-bpF)>(bpS-preRunPos): #current SNP position is too far from the end of the gene vs prev 
                    fOut.write("%s %i\n"%(gene,indx-1))
                else:   #current SNP is closer to the gene than previous one
                    fOut.write("%s %i\n"%(gene,indx))
                
    fIn.close()
    fBase.close()
    fOut.close()

def filterVar(fName):
 '''
 ##   given *.tped file name filters out non-variant SNPs and leaves only those that passed MAC filter
 '''
 fIn = open(fName,'r')
 fOut= open(fName.replace(".tped","_v%i.tped"%MAC_FILTER),'w')
 for i in fIn:
    l=i.replace("\n","").split()
    count=[0,0]
    for j,k in enumerate(l):
        if j>3:
            if 'A' in k:
                count[0]+=0.5
            elif 'B' in k:
                count[1]+=0.5
    if min(count)>=MAC_FILTER:
        fOut.write(i)
 fIn.close()
 fOut.close()

#######################
def checkConfig4Run():
    '''
    checks if all files are in order for GWA run
    '''
    #confirm the loaded data
    bPass = True
    if not os.path.isdir(GENO_LOC):
        print 'E1. Genome location provided in config file is not valid.--%s'%(GENO_LOC)
        bPass=False

    PhFile = os.path.join(PHENO_LOC,PHENO_FILE)  #original phenotypic file
    if not os.path.exists(PhFile):
        print 'E2. Pheno data file provided in config file does not exist.--%s'%(PhFile)
        bPass=False
    else:
        nameBase=PHENO_FILE.split('.')[0]           #common name for the pheno traits group (based on the file name)
    if len(POPS)==0:
        print 'E3. List of sub-populations could nod be empty, in case of processing all lines, use ALL name'
        bPass=False
    if len(GROUPS)==0:
        print 'E4. List of RDP groups could nod be empty, in case of processing both groups, use RDP12 name'
        bPass=False
    nameBase=PHENO_FILE.split('.')[0]
    for P in POPS:
        for G in GROUPS:
            if not os.path.exists("%spheno%s_%s_%s_%s.txt"%(PHENO_LOC,nameBase,GT_VER,G,P)): #Check pheno file existance
                bPass=False
                print 'E5. Sub-pop phenotypic file does not exist.'
            if not os.path.exists("%s%s_%s_%s_%s_maf%s_v%s.h%s.kinf"%(GENO_LOC, nameBase, GT_VER, G, P, MAF_FILTER, MAC_FILTER,MATRIX)): #kinship matrix check
                bPass=False
                print 'E6. No kinship matrix found.'
            if int(PC)>0: 
                if not os.path.exists("%spc_cov%s_%s_%s_%s_%s_maf%s_v%s.txt"%(GENO_LOC, PC, nameBase, GT_VER, G, P, MAF_FILTER, MAC_FILTER)):
                    bPass=False
                    print 'E6. No PC co-variate matrix found. :%s'%"%spc_cov%s_%s_%s_%s_%s_maf%s_v%s.txt"%(GENO_LOC, PC, nameBase, GT_VER, G, P, MAF_FILTER, MAC_FILTER)
    if not os.path.isdir(R_OUT_LOC):
        runCommand("mkdir %s"%R_OUT_LOC)
    if not os.path.isdir(PLOT_LOC):
        runCommand("mkdir %s"%PLOT_LOC)
    return bPass

######################################################
def checkConfig4Prep():
    '''
    checks if all files are in order for files preparation run
    '''
    #confirm the loaded data
    bPass = True
    if not os.path.isdir(GENO_LOC):
        print 'P1. Genome location provided in config file is not valid.--%s'%(GENO_LOC)
        bPass=False
    PhFile = os.path.join(PHENO_LOC,PHENO_FILE)  #original phenotypic file
    if not os.path.exists(PhFile):
        print 'P2. Pheno data file provided in config file does not exist.--%s'%(PhFile)
        bPass=False
    else:
        nameBase=PHENO_FILE.split('.')[0]           #common name for the pheno traits group (based on the file name)
    if len(POPS)==0:
        print 'P3. List of sub-populations could nod be empty, in case of processing all lines, use ALL name'
        bPass=False
    if len(GROUPS)==0:
        print 'P4. List of RDP groups could nod be empty, in case of processing both groups, use RDP12 name'
        bPass=False
    GS=os.path.join(GENO_LOC,GFILE)   #original plink genome file
    if not (os.path.exists("%s.map"%GS))&(os.path.exists("%s.ped"%GS)):
        print 'Genome files were not found: %s or %s'%("%s.map"%GS,"%s.ped"%GS)
        bPass=False
    return bPass

def gSNP_Allele_info():
    '''
    using SNPs informationfile creates a dictionary of SNPs where the key is SNP name, first element - chromosome, second - position
    A - allele, B - allele
    '''
    dOut=dict()
    if not os.path.exists(SNPsDATA):
        print " No SNPs information file found : %s"%SNPsDATA
    else:
        fIn=open(SNPsDATA,'r')
        for i in fIn:
            l=i.split()
            dOut[l[0]]={'chrom':l[1].split('Chr')[-1] , 'pos':int(l[2]) , 'allel':{'A':l[3],'B':l[4]} }
        fIn.close()
    return dOut

def checkPemmaxLimit(fName, pLimit):
    '''
    #
    #given a result file and pValue limit, returns a list of sorted by pValue SNPs with a pValue elements that satisfy limit condition
    #
    '''
    fIn=open(fName,'r')
    SNP=[]
    SNPList=[]
    for i in fIn:
        SNP.append(i.replace('\n','').split())
        SNPList.append(i.replace('\n','').split()[0])
    fIn.close()
    snpArray = np.array(SNP)

    PList=snpArray[:, [2]] #get p-values
    PList=np.where(PList != 'NA', PList, 1).astype(float) #replace NA with value 1
    dSNP=dict()
    iCount=0
    x = np.zeros((len(PList),2),dtype=float)

    for row in PList:
        x[iCount][0]=iCount
        x[iCount][1]=row[0]
        iCount+=1
    xSort=x[x[:,1].argsort()] 
    iCount=0
    sOut=[]
    for S in xSort:
        if S[1]<pLimit:
            iCount+=1
            sOut.append({'snp':SNPList[int(S[0])],'pV':S[1]})
        else:
            break
    print "%i SNPs found in %s file"%(iCount,fName)
    return sOut

##############################
def extractPhenoFreqDta(lsSNPs, lTraitData, dSNP, GFile, MFile):
    #gets Allele frequencies and mean values for each of the SNP in the list
    #first get a list of idexesSNPs
    #print lTraitData
    fIn=open(MFile,'r')
    lSNP=list()
    ldData=list()
    for i in fIn:
        lSNP.append(i.split()[1])
    fIn.close()
    
    lIndx=list()               #collect index for each SNP
    for SNP in lsSNPs:
        if not SNP['snp'] in lSNP:
            print '%s not found in like: %s or %s'%(SNP['snp'],lSNP[0],lSNP[1])
        lIndx.append(lSNP.index(SNP['snp']))
        ldData.append({})

    fIn=open(GFile,'r')
    iCount=0
    for i in fIn:
        l=i.replace("\n","").split()
        for j,ind in enumerate(lIndx):
            A1 = l[6+2*ind]  
            A2 = l[7+2*ind]
            if A1!='0' and  A2!='0' and lTraitData[iCount]!='NA': #although if one is missing  - second suppose to be missing too
                AA="%s%s"%(dSNP[lsSNPs[j]['snp']]['allel'][A1] , dSNP[lsSNPs[j]['snp']]['allel'][A2])
                if AA in ldData[j]:
                    ldData[j][AA].append(float(lTraitData[iCount]))
                else:
                    ldData[j][AA]=[float(lTraitData[iCount])]
        iCount+=1

    dOut=dict()
    for j,k in enumerate(ldData):
        iMajor=0
        iMinor=0
        kMajor='0'
        kMinor='0'
        kGetero='0'
        iGetero=0
        keys=k.keys()
        if len(keys)>3:
            print k
        keys.sort()
        for m in keys:
            if m[0]==m[1]: #heterozygote key
                if len(k[m])>=iMajor:
                    if iMajor!=0:
                       iMinor=iMajor
                       kMinor=kMajor
                    iMajor=len(k[m]) 
                    kMajor=m
                else:
                    iMinor=len(k[m])
                    kMinor=m
            else:
                kGetero=m
                iGetero=len(k[m])

        iTote=iMajor+iMinor+iGetero
        if iGetero==0:
            getAv=0
        else:
            getAv=float(sum(k[kGetero]))/len(k[kGetero])
        majAv=float(sum(k[kMajor]))/len(k[kMajor])
        if iMinor==0:
            minAv=0
        else:
            minAv=float(sum(k[kMinor]))/len(k[kMinor])
        dOut[lsSNPs[j]['snp']]={}
        dOut[lsSNPs[j]['snp']]['MajA']=kMajor[0]
        dOut[lsSNPs[j]['snp']]['MinA']=kMinor[0]
        dOut[lsSNPs[j]['snp']]['MajA_count']=iMajor 
        dOut[lsSNPs[j]['snp']]['MajA_pcnt']=100*float(iMajor)/iTote
        dOut[lsSNPs[j]['snp']]['MajA_T_Av']=majAv
        dOut[lsSNPs[j]['snp']]['Get_count']=iGetero
        dOut[lsSNPs[j]['snp']]['Get_pcnt']=100*float(iGetero)/iTote
        dOut[lsSNPs[j]['snp']]['Get_T_Av']=getAv
        dOut[lsSNPs[j]['snp']]['MinA_count']=iMinor
        dOut[lsSNPs[j]['snp']]['MinA_pcnt']=100*float(iMinor)/iTote
        dOut[lsSNPs[j]['snp']]['MinA_T_Av']=minAv
        dOut[lsSNPs[j]['snp']]['AllEffect']=majAv-minAv
    return dOut  

def prep_files():
 '''
 # Prepares files for GWAS:
 #    1. Makes pheno files for each sub-population specified in config file matched to geno files
 #    2. Creates and fiters genome files for each sub-population specified in config file
 #    3. Makes kinship matrix file and covariate file if requested
 '''
 init()    #read config file
 if not checkConfig4Prep(): #confirm the loaded data
    print "Not all files are available for the file preparation run. Check the appconfig file"
    exit
 SB=POPS      #sub-poulations
 GS=os.path.join(GENO_LOC,GFILE)   #original plink genome file

 if 'ALL' in SB:
    bALL=True   #group ALL presented in the list - make it to be the last in the list
    SB.sort(reverse=True)

 #collect sub-pop and RDP information
 fLData=open(POPFILE,'r')
 dID=dict()
 for i in fLData:
    l=i.replace("\n","").split("\t")
    dID[l[0]]=[l[1],l[2]]
 fLData.close()
 print "%i SUB POS info found"%len(dID)
 PhFile = os.path.join(PHENO_LOC, PHENO_FILE)  #original phenotypic file
 nameBase=PHENO_FILE.split('.')[0]
 for G in GROUPS:   #loop through RDP groups
     header=True
     fPhIn=open(PhFile,'r') #original pheno traits file
     sIdx=list()
     sFOut=list()   #list of new sub-population genome file handlers 
     ssLines=list()   #list of sub-population lists

     for S in SB:
         sFOut.append(open("%spheno%s_%s_%s_%s.txt"%(PHENO_LOC, nameBase, GT_VER, G, S),'w')) 
         ssLines.append({})
     iC=0
     for i in fPhIn:
         if header:
             header = False
             for s,S in enumerate(SB):
                 sFOut[s].write(i) #write headers  
         else:
             l=i.replace("\n","").split()
             if l[0] in dID: #check if sub-pop info exists for the line
                 if dID[l[0]][1]==G or G=="RDP12":
                     iC+=1
                     for j,k in enumerate(POPS):
                         if dID[l[0]][0] == k:  #add to single pops
                             ssLines[j][l[0]]=i
                     if bALL:
                         ssLines[-1][l[0]]=i #write to ALL pop
             else:
                 print "%s line not found in the dictionary"%l[0]
     print "%i sub identified lines found"%iC

     fPhIn.close()  #original pheno file handler close    
     
     sFGOut=list()  #list of new sub-population genome file handlers
     GFIn=open("%s.ped"%GS,'r')
 
     #making new ped files for each sub-population
     for S in SB:
         sFGOut.append(open("%s%s_%s_%s_%s.ped"%(GENO_LOC,nameBase,GT_VER,G,S),'w'))
         
     for i in GFIn: #loop through original genome file records
         l=i.split()
         for j,t in enumerate(SB):
             if l[1] in ssLines[j]: # subs dictionary
                 sFGOut[j].write(i)
                 sFOut[j].write(ssLines[j][l[1]])

     for t,S in enumerate(SB):
         sFGOut[t].close()     #closing new sub-pop genome files (.ped)
         sFOut[t].close()      #new pheno files hanlers closing

     GFIn.close() 
 
     for S in SB:              ##   phenotestData_G6_4_RDP1_ALL.txt
         runCommand('%splink --map %s.map --ped %s%s_%s_%s_%s.ped --recode transpose --maf %s --geno %s --out %s%s_%s_%s_%s_maf%s --noweb'%(PLINK, GS, GENO_LOC, nameBase, GT_VER, G, S, MAF_FILTER, GENO_FILTER, GENO_LOC, nameBase, GT_VER, G, S, MAF_FILTER))
         filterVar("%s%s_%s_%s_%s_maf%s.tped"%(GENO_LOC, nameBase, GT_VER, G, S, MAF_FILTER))   #apply MAC filter 
         runCommand('%splink --tfam %s%s_%s_%s_%s_maf%s.tfam --tped %s%s_%s_%s_%s_maf%s_v%s.tped --recode --out %s%s_%s_%s_%s_maf%s_v%s --noweb'%(PLINK, GENO_LOC, nameBase, GT_VER, G, S, MAF_FILTER, GENO_LOC, nameBase, GT_VER, G, S, MAF_FILTER, MAC_FILTER, GENO_LOC, nameBase, GT_VER, G, S, MAF_FILTER, MAC_FILTER))
         runCommand("%splink --file %s%s_%s_%s_%s_maf%s_v%s --recode12 --transpose --out %s%s_%s_%s_%s_maf%s_v%s --noweb"%(PLINK, GENO_LOC, nameBase, GT_VER, G, S, MAF_FILTER, MAC_FILTER, GENO_LOC, nameBase, GT_VER, G, S, MAF_FILTER, MAC_FILTER))
         if MATRIX=="IBS":
             runCommand("%semmax-kin -v -h -s -d 10 %s%s_%s_%s_%s_maf%s_v%s"%(EMM_LOC,  GENO_LOC, nameBase, GT_VER, G, S, MAF_FILTER, MAC_FILTER))
         elif MATRIX=="BN":
             runCommand("%semmax-kin -v -h -d 10 %s%s_%s_%s_%s_maf%s_v%s"%(EMM_LOC,  GENO_LOC, nameBase, GT_VER, G, S, MAF_FILTER, MAC_FILTER))
         if int(PC)>0: #PC covariate file requested
             runCommand("%splink --file %s%s_%s_%s_%s_maf%s_v%s --recode A --out %s%s_%s_%s_%s_maf%s_v%s --noweb"%(PLINK, GENO_LOC, nameBase, GT_VER, G, S, MAF_FILTER, MAC_FILTER, GENO_LOC, nameBase, GT_VER, G, S, MAF_FILTER, MAC_FILTER))
             runCommand("Rscript %s %s %s%s_%s_%s_%s_maf%s_v%s %spheno%s_%s_%s_%s.txt %s_%s_%s_%s_maf%s_v%s %s %s"%(RScript4PC, '1', GENO_LOC, nameBase, GT_VER, G, S, MAF_FILTER, MAC_FILTER,PHENO_LOC,nameBase,GT_VER,G,S,nameBase, GT_VER, G, S, MAF_FILTER, MAC_FILTER, GENO_LOC, PC))
      
  



def runGWA():      
 '''
 # run GWAS:
 #    1. Checks if all needed file exist
 #    2. Runs GWA using EMMAX mixed model
 #    2. Makes QQ and MP for each trait
 #    3. Makes summary table for the set of most significant SNPs
 '''
 if not checkConfig4Run():  #check required files existance
    print "Not all files are available for the run. Please call prep_files script first"
    exit

 leastP = 1
 resFile='%sresults_summary.txt'%R_OUT_LOC
 fOut=open(resFile,'w')
 BN=PHENO_FILE.split('.')[0] #Name base for the output files 
 fSNPOut=open('%sbest_SNPs_summary_list_%s_%s.csv'%(R_OUT_LOC, BN, GT_VER),'w')
 fSNPOut.write("TRAIT\tGROUP\SUB_POP\tCOV_PC\tFILTER\tSNP-NAME\tCHROMOSOME\tPOSITION\tpVALUE\tMajorAllele\tMinorAllele\tMajorAllele_Count\tMajorAllele_%%\tMajorAllele_Trait_Average\tGetero_Count\tGetero%%\tGeteroTrait_Average\tMinorAllele_Count\tMinorAllele%%\tMinorAllele_Trait_Average\tAllele_Effect\n")

 dSNP=gSNP_Allele_info()
 if len(dSNP)==0:
    print "No SNPs information file found. Set proper SNPsDATA parameter in GenomeData part of the configuration file."
    exit

 for G in GROUPS:
  B="%s_%s_%s"%(G, GT_VER, BN) 
  for Pop in POPS:
   PH_ORIG = "%spheno%s_%s_%s_%s.txt"%(PHENO_LOC, BN, GT_VER, G, Pop)  
   lP = getHeaders(PH_ORIG) #get a list of traits
   for m,n in enumerate(lP):
       if '%' in n:
           lP[m]=n.replace('%','proc')        
   GEN_F="%s_%s_%s_%s_maf%s_v%s"%(BN, GT_VER, G, Pop, MAF_FILTER, MAC_FILTER)
   
   for j,p in enumerate(lP): 
          IBD_F="%s%s.h%s.kinf"%(GENO_LOC, GEN_F, MATRIX)
          if PC=='0':
              PCCov_F = "NA"
          else:  
              PCCov_F = "%spc_cov%s_%s.txt"%(GENO_LOC, PC, GEN_F)    
     
          print("processing %s phenotype with %s matrix and %s covariate file for %s pop with %s base"%(p, MATRIX, PCCov_F, Pop, B))
          base = "%s_%s_%s_c%s"%(p, GEN_F, MATRIX, PC)
          baseOut = "%s%s"%(R_OUT_LOC, base)
          res_file = '%s.ps'%baseOut
          prelim_file = '%s_prelim'%baseOut
          pFile,lTraitData=makeSinglePhenoFileList_ExH(PH_ORIG, j, p)
          if not os.path.isfile(res_file) or OVERWRITE: #to prevent re-processing if not instructed to do so
              run_EMMAX('%s%s'%(GENO_LOC,GEN_F), pFile, IBD_F, baseOut, PCCov_F)
          runCommand("rm %s"%pFile)  #remove temporary single trait phenofile
          lSNP=checkPemmaxLimit(res_file, BEST_SNP_LIMIT)
          dSNP_PF=extractPhenoFreqDta(lSNP, lTraitData, dSNP, '%s%s.ped'%(GENO_LOC, GEN_F), '%s%s.map'%(GENO_LOC, GEN_F))
          for SNP in lSNP:
              if SNP['snp'] in dSNP:
                  fSNPOut.write("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%i\t%e\t%s\t%s\t%i\t%f\t%f\t%i\t%f\t%f\t%i\t%f\t%f\t%f\n"%(p, G, Pop,"%sPC"%PC,'maf%s_mac%s'%(MAF_FILTER, MAC_FILTER), SNP['snp'],dSNP[SNP['snp']]['chrom'],dSNP[SNP['snp']]['pos'],SNP['pV'],dSNP_PF[SNP['snp']]['MajA'],dSNP_PF[SNP['snp']]['MinA'],dSNP_PF[SNP['snp']]['MajA_count'],dSNP_PF[SNP['snp']]['MajA_pcnt'],dSNP_PF[SNP['snp']]['MajA_T_Av'],dSNP_PF[SNP['snp']]['Get_count'],dSNP_PF[SNP['snp']]['Get_pcnt'],dSNP_PF[SNP['snp']]['Get_T_Av'],dSNP_PF[SNP['snp']]['MinA_count'],dSNP_PF[SNP['snp']]['MinA_pcnt'],dSNP_PF[SNP['snp']]['MinA_T_Av'],dSNP_PF[SNP['snp']]['AllEffect']))
              else:
                  print "%s SNP was not found in the dictionary"%SNP[0]
          minP=lSNP[0]['pV']
          minSNP=lSNP[0]['snp']
          
          fOut.write('%s have min P val for %s assoc: %e\n'%(minSNP, baseOut, minP))
          if mkQQplot=='1':
              runCommand('Rscript %s %s %s 3 %s %s %s %s %s %s'%(RScript4PC, '2', res_file, p, B, '%s_%s_%s_pc%s'%(Pop,MATRIX,'maf%s_mac%s'%(MAF_FILTER, MAC_FILTER),PC) , '1', '1', PLOT_LOC))
          if mkMPlot=='1':
              dEdges,nChr,fAxes = getEdgesOrdered(res_file, base, R_OUT_LOC, P_FILTER, dSNP) 
              fMP,nP = makeMPfinal(base, dEdges, R_OUT_LOC)
              runCommand('Rscript %s %s %s %i %s %i %s %s %s %s %s %s %s %s %s'%(RScript4PC, '3', fMP, nP, fAxes, len(dEdges), p, Pop, base, "%s-%s"%(G, GT_VER),"0","Rice", FDR_LIMIT, "%sPC"%PC, PLOT_LOC))
              runCommand('rm %saxes*'%R_OUT_LOC)
              runCommand('rm %sMP_*'%R_OUT_LOC)
              runCommand('rm %s*_edges'%R_OUT_LOC)
              runCommand('rm %s*_prelim'%R_OUT_LOC)
 fOut.close()
 fSNPOut.close()                

#####################################################
######### Main Program
#########
######### 1. Loads data from configuration file
######### 2. Prepair files for GWA
######### 2. Runs GWA
######### 3. Build QQ and MPlots
####################################################    


print "Main"

init()   #loading configuration data 
if not checkConfig4Run(): #check files for GWA
    prep_files()          #prepair files for GWA
runGWA()                  #runs GWA and builds plots
